# this file ONLY contains graph_to_ideal, and is meant to be
# loaded from other files
# via the command load('func_graph_to_ideal.sage')
# see the file experiments.sagews for an example
#
# DESC: graph_to_ideal takes as input a graph (and related information)
#     and returns as output the associated perfect matching ideal
#
# INPUT:
#     g: a object of sage graph class
#
# OUTPUT:
#    P: a F_2[x_1,...x_e], a polynomial ring over F2 with one variable per edge
#    I: a list of the polynomials in the perfect matching ideal
#
def graph_to_ideal(g,o,r):
    print "Entering graph_to_ideal... "
    #print "G: " + str(G)
    print "num_vertices = " + str(g.order())
    print "num_edges = " + str(g.size())
    
    P = PolynomialRing(FiniteField(2), g.order(), 'x', order="lex")
    #I = list(); # create an empty list
    
    # the incidence matrix is a (# of vertices) x (# of edges matrix)
    # where A(i,j) = 1 if vertex i is incident on edge j, and 0 otherwise
    A = g.adjacency_matrix()
    #print str(A)
    # this is just for debugging
    debug = 1
    if debug == 1:
        count = 0;
        for e in g.edge_iterator():
            #print "x[" + str(count) + "]: " + str(e)
            count = count + 1;
        #end for
    #end if debug

    if o==0 and r==0:
        # establish the ring containing the ideal (this will be returned)
        # number of variables is number of edges
        P = PolynomialRing(FiniteField(2), g.order(), 'x', order="lex") # over F2
        x = P.gens();
        #print str(type(P))
        I = list(); # create an empty list
        # for each vertice, create polynomial x^3-1
        for i in range(0, g.order()): # for each vertex in the graph, cr
            I.append(x[i]^2-x[i])
            #print("I: ") + str(I)
            #print "Constructing polynomials, one per vertice" + str(x[i]^3-1)
            for j in range(i+1, g.order()):
                if A[i,j] == 1:
                    I.append(x[i]+x[j]-1)
                    #print("I: ") + str(I)

    if o==0 and r==1:
            # establish the ring containing the ideal (this will be returned)
        # number of variables is number of edges
        P = PolynomialRing(CC, g.order(), 'x', order="lex") # over the rationals
        x = P.gens();
        #print str(type(P))
        I = list(); # create an empty list
        #print "Constructing boolean polynomials, one per edge"
        # for each edge, create the polynomial eij^2 - eij
        for i in range(0, g.order()): # for each vertex in the graph
            I.append(x[i]^2-x[i])
            for j in range(i+1, g.order()):
                if A[i,j] == 1:
                    I.append(x[i]+x[j]-1)
                    #
                    #endif
                #end for ee
            #endif
        #end for edges
    #end for vertices

    if o==1 and r==0:
            # establish the ring containing the ideal (this will be returned)
            # number of variables is number of edges
        P = PolynomialRing(FiniteField(2), g.order(), 'x', order="deglex") # over F2
        x = P.gens();
            #print str(type(P))
        I = list(); # create an empty list
            #print "Constructing boolean polynomials, one per edge"
        # for each edge, create the polynomial eij^2 - eij
        for i in range(0, g.order()): # for each vertex in the graph
            I.append(x[i]^2-x[i])
            for j in range(i+1, g.order()):
                if A[i,j] == 1:
                    I.append(x[i]+x[j]-1)
                    #
                    #endif
                #end for ee
            #endif
        #end for edges
    #end for vertices

    if o==1 and r==1:
            # establish the ring containing the ideal (this will be returned)
        P = PolynomialRing(CC, g.order(), 'x', order="deglex") # over the rationals
        x = P.gens();
            #print str(type(P))
        I = list(); # create an empty list
            #print "Constructing boolean polynomials, one per edge"
            # for each edge, create the polynomial eij^2 - eij
        for i in range(0, g.order()): # for each vertex in the graph
            I.append(x[i]^2-x[i])
            for j in range(i+1, g.order()):
                if A[i,j] == 1:
                    I.append(x[i]+x[j]-1)
                    #
                    #endif
                #end for ee
            #endif
        #end for edges
    #end for vertices
            
    if o==2 and r==0:
            # establish the ring containing the ideal (this will be returned)
            # number of variables is number of edges
        P = PolynomialRing(FiniteField(2), g.order(), 'x', order="degrevlex") # over F2
        x = P.gens();
            #print str(type(P))
        I = list(); # create an empty list
            #print "Constructing boolean polynomials, one per edge"
            # for each edge, create the polynomial eij^2 - eij
        for i in range(0, g.order()): # for each vertex in the graph
            I.append(x[i]^2-x[i])
            for j in range(i+1, g.order()):
                if A[i,j] == 1:
                    I.append(x[i]+x[j]-1)
                    #
                    #endif
                #end for ee
            #endif
        #end for edges
    #end for vertices

    if o==2 and r==1:
        P = PolynomialRing(CC, g.order(), 'x', order="degrevlex")
        x = P.gens();
        print str(type(P))
        I = list(); # create an empty list
        print("ideal")
            #print "Constructing boolean polynomials, one per edge"
            # for each edge, create the polynomial eij^2 - eij
        for i in range(0, g.order()): # for each vertex in the graph
            I.append(x[i]^2-x[i])
            for j in range(i+1, g.order()):
                if A[i,j] == 1:
                    I.append(x[i]+x[j]-1)
                    #
                    #endif
                #end for ee
            #endif
        #end for edges
    #end for vertices


    # use the "dictionary" method return the variables
    return {'P':P, 'I':I}
#enddef graph_to_ideal